/* enpyct.c */
/* Copyright 2002 Michael Daum */

#include <png.h>
#include <stdio.h>

png_bytep row_data;
png_structp png_ptr;
png_infop info_ptr;
png_uint_32 width, height;
png_bytep * row_pointers;
char msgbuf[1024];

int main(int argc, char **argv)
{
        if ( argc < 2){
                fprintf(stderr,"Usage %s png_to_enpyct [outfile]\n",argv[0]);
                exit(1);
        }
        read_png(argv[1]);
        munge_png();
        write_png((argc > 2)?argv[2]:"out.png");
}

int munge_png()
{
        png_bytep p = row_data + 16;
        unsigned int num_read;
        unsigned int total = 0;
        unsigned int c_offset;
        unsigned char *cp = (unsigned char *)&total;
        /* enpyct the bits */
        while( num_read = read(0,msgbuf,1024) ){
                if (total + num_read > width * height - 16){
                        fprintf(stderr,"Too much data, can encode %d bytes in this image\n",width * height - 16);
                        fprintf(stderr,"Try a larger image\n");
                        exit(1);
                }
                for (c_offset = 0; c_offset < num_read ; c_offset++){
                        *p = (*p & 0xfc) | ((msgbuf[c_offset] >> 6) & 0x03);
                        p++;
                        *p = (*p & 0xfc) | ((msgbuf[c_offset] >> 4) & 0x03);
                        p++;
                        *p = (*p & 0xfc) | ((msgbuf[c_offset] >> 2) & 0x03);
                        p++;
                        *p = (*p & 0xfc) | ( msgbuf[c_offset]       & 0x03);
                        p++;
                        total += 1;
                }
        }
        /* write the data length into bytes 0x00..0x0f */
        p = row_data;
        for (c_offset = 0; c_offset < 4 ; c_offset++){
                *p = (*p & 0xfc) | ((cp[c_offset] >> 6) & 0x03);
                p++;
                *p = (*p & 0xfc) | ((cp[c_offset] >> 4) & 0x03);
                p++;
                *p = (*p & 0xfc) | ((cp[c_offset] >> 2) & 0x03);
                p++;
                *p = (*p & 0xfc) | ( cp[c_offset]       & 0x03);
                p++;
        }        
}

int read_png(char *file_name)
{
   unsigned int sig_read = 0;
   int bit_depth, color_type, interlace_type;
   FILE *fp;
   int row;

   if ((fp = fopen(file_name, "r")) == NULL)
           return -1;
   
   png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,
                                    NULL,NULL,NULL);
   
   if (png_ptr == NULL)
   {
           fclose(fp);
           return -1;
   }
   
   /* Allocate/initialize the memory for image information.  REQUIRED. */
   info_ptr = png_create_info_struct(png_ptr);
   if (info_ptr == NULL)
   {
           fclose(fp);
           png_destroy_read_struct(&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
           return -1;
   }
   
   /* Set error handling if you are using the setjmp/longjmp method (this is
    * the normal method of doing things with libpng).  REQUIRED unless you
    * set up your own error handlers in the png_create_read_struct() earlier.
    */
   
   if (setjmp(png_jmpbuf(png_ptr)))
   {
           /* Free all of the memory associated with the png_ptr and info_ptr */
           png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
           fclose(fp);
           /* If we get here, we had a problem reading the file */
           return -1;
   }
   
   png_init_io(png_ptr, fp);

   png_set_sig_bytes(png_ptr, sig_read);

   png_read_info(png_ptr, info_ptr);

   png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type,
       &interlace_type, NULL, NULL);

   png_set_strip_16(png_ptr);
   png_set_packing(png_ptr);
   if (color_type == PNG_COLOR_TYPE_PALETTE)
      png_set_expand(png_ptr);
   if (color_type == PNG_COLOR_TYPE_GRAY && bit_depth < 8)
      png_set_expand(png_ptr);
   if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
      png_set_expand(png_ptr);
   png_set_filler(png_ptr, 0xff, PNG_FILLER_AFTER);
   png_read_update_info(png_ptr, info_ptr);

   fprintf(stderr,"%d x %d\n",width, height);
   row_data = (png_bytep)malloc(height * png_get_rowbytes(png_ptr,info_ptr));
   row_pointers = (png_bytep *)malloc(height * sizeof(png_bytep));
   for ( row = 0 ; row < height; row++){
           row_pointers[row] = row_data + row * png_get_rowbytes(png_ptr,info_ptr);
   }
   png_read_image(png_ptr, row_pointers);

/*   png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL); */

   fclose(fp);

   /* that's it */
   return 0;

}

int write_png(char *file_name)
{
   FILE *fp;

   fp = fopen(file_name, "w");
   if (fp == NULL)
           return -1;

   png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,
                                     NULL,NULL,NULL);
   
   if (png_ptr == NULL)
   {
           fclose(fp);
           return -1;
   }

   if (setjmp(png_jmpbuf(png_ptr)))
   {
      /* If we get here, we had a problem reading the file */
      fclose(fp);
      png_destroy_write_struct(&png_ptr, &info_ptr);
      return -1;
   }

   png_init_io(png_ptr, fp);

   png_set_IHDR(png_ptr, info_ptr, width, height,8, PNG_COLOR_TYPE_RGB_ALPHA,
      PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);

   png_write_info(png_ptr, info_ptr);
   png_write_image(png_ptr, row_pointers);

   fclose(fp);

   return 0;
}


